/*
 * @(#)FileCopy.java	1.00 97/02/23
 *
 * Copyright (c) 1997 Fabula Software GmbH. All Rights Reserved.
 *
 * <a href=http://www.fabula.com>Fabula Software GmbH</a>
 * <a href=mailto:info@fabula.com>eMail</a>
 */

package fabula.io;

import  java.io.*;
import  java.util.*;

/**
 * A class for copy files
 * <p>
 *
 * @version 	1.00, 23 Feb 1997
 * @author	Stefan Bther
 */

public class FileCopy {

  /**
   * BLOCKSIZE is the buffer size which is used during copying
   */
  private static int BLOCKSIZE = 1024;

  public  int BytesComplete;
  public  int BytesWritten;

  /**
   * Copy a file with File objects.
   * @aSource is the source file, @aTarget specify where to copy the file
   * @see     java.io.File
   */
  public FileCopy(File aSource, File aTarget) throws IOException {

		FileInputStream  aSrcStream = null;
		FileOutputStream aTargetStream = null;
		byte[] aBuffer;
		int aBytesRead;

    BytesWritten  = 0;
    BytesComplete = 0;

   	if (! aSource.exists())
 	     throw new IOException("Source not exists: " + aSource);

    if (! aSource.canRead())
       throw new IOException("Source can't read: " + aSource);

    aSrcStream = new FileInputStream(aSource);
    aTargetStream = new FileOutputStream(aTarget);
    aBuffer = new byte[BLOCKSIZE];

    // BytesComplete = aSrcStream.

    try {
      while (true) {
        aBytesRead = aSrcStream.read(aBuffer);
        if (aBytesRead == -1)
           break;
        aTargetStream.write(aBuffer,0,aBytesRead);
        BytesWritten += aBytesRead;
      }
    }
    finally {
      aSrcStream.close();
      aTargetStream.close();
    }
  }

  /**
   * Copy a file with String objects.
   * @aSource is the source file, @aTarget specify where to copy the file
   * @see     java.io.File
   */
  public FileCopy(String aSource, String aTarget ) throws IOException {
    this(new File(aSource), new File(aTarget));
  }
}


