/*
 * @(#)Annuity.java	1.00 97/02/23
 *
 * Copyright (c) 1997 Fabula Software GmbH. All Rights Reserved.
 *
 * <a href=http://www.fabula.com>Fabula Software GmbH</a>
 * <a href=mailto:info@fabula.com>eMail</a>
 */

package fabula.util;

/**
 * The class <code>Annuity</code> calculates the "Effektivzins" after 
 * the annuity method. The entry value for this calculation must be 
 * first calculated by the Uniform method which is also included. 
 * <p>
 *
 * @author  <a href="mailto:stefc@fabula.com">Stefan Bther</a>
 * @company <a href="http://www.fabula.com">Fabula Software</a>
 * @version 1.00, 31 Mar 1997
 */

public class Annuity {

	/**
     * Uniformmethode um einen Nherungswert zu ermitteln
     */
	public static double Uniform( double p, double b, int M ) {

		if ( M==0) { 
			return 0.0;
		} else {
			return 24 * (((p*M)+b)/(M+1));
		}
	}


	/**
     * Annuitten Methode um genauer zu rechnen 
     */
	public static double Calc( double p, double b, int M, double e) {


		if ( e < 0.001) return 0.0; 

		int j = M / 12;
		int rm = M - (12 * j);
		int Count = 1;

		double Delta = 1.0; 
		double a1 = ((1+(b/100))/M) + (p/100);
		double x = 2;
		double i,q,q_j,a2;

		while ( Math.abs(Delta) > 1E-10) {
			Count++;		
			if (Count > 1000) break;  // iterative border if no convergence 

			i = e / 100;
			q = 1+i;
			q_j = Math.pow(q,j); 
			a2 = q_j / (
					(((12/i)+5.5) * (q_j-1)) +
					((1+((rm-1)/24)*i) * (rm/(1+(rm/12)*i)))
				);
			Delta = (a1-a2);

			if (Delta>0) {
				e = e + x; 
			} else { 
				e = e - x;
			}

			x  = x / 2;
		}

		return e;
	}

}
