/*
 * @(#)Rect.java	
 *
 * Copyright (c) 1997 Fabula Software GmbH. All Rights Reserved.
 *
 * <a href=http://www.fabula.com>Fabula Software GmbH</a>
 * <a href=mailto:info@fabula.com>eMail</a>
 */

package fabula.util;

import  java.awt.*;

/**
 * The class <code>Rect</code> defines several methods for handling 
 * rectangles easy
 * <p>
 *
 * @author  <a href="mailto:stefc@fabula.com">Stefan Bther</a>
 * @company <a href="http://www.fabula.com">Fabula Software</a>
 * @version 1.00, 31 Mar 1997
 */

public class Rect {

	/**
	 * Don't let anyone instantiate this class.
	 */
	private Rect() {}


	/**
     * returns the height of the rectangle 
     */
	public static int Height(Rectangle R) {
		return R.height;
	}

	/**
     * returns the widht of the rectangle 
     */
	public static int Width(Rectangle R) {
		return R.width;
	}

	/**
     * returns the left position of the rectangle 
     */
	public static int Left(Rectangle R) {
		return R.x;
	}

	/**
     * returns the top position of the rectangle 
     */
	public static int Top(Rectangle R) {
		return R.y;
	}

	/**
     * returns the right position of the rectangle 
     */
	public static int Right(Rectangle R) {
		return R.x+R.width;
	}

	/**
     * returns the bottom position of the rectangle 
     */
	public static int Bottom(Rectangle R) {
		return R.y+R.height;
	}

	/**
     * grows the rectangle 
     */
	public static Rectangle Grow( Rectangle R, int Delta ) {
		R.grow(Delta, Delta);
		return R;
	}

	/**
     * relative move of the rectangle 
     */
	public static Rectangle RelativeMove( Rectangle R, int dx, int dy) {
		R.translate(dx,dy);
		return R;
	}
	
	/**
     * absolute move of the rectangle 
     */
	public static Rectangle MoveTo( Rectangle R, int x, int y) {
		R.move(x,y);
		return R;
	}

	/**
     * build a rectangle in Windows notation 
     */
	public static Rectangle Set ( int Left, int Top, int Right, int Bottom ) {
		return new Rectangle(Left, Top, Right-Top, Bottom-Top);
	}

	/**
     * Create a rectangle with given two points 
     */
	public static Rectangle SetPoint( Point TopLeft, Point BottomRight) {
		return new Rectangle(TopLeft, 
			 new Dimension(BottomRight.x-TopLeft.x, BottomRight.y-TopLeft.y));
	}

	/**
     * check if the two rectangles intersects 
     */
	public static boolean Include(Rectangle R1, Rectangle R2) {
		return R1.intersects(R2);
	}

	/**
     * checks if the point is inside the rectangle 
     */
	public static boolean Point(Rectangle R, Point P ) {
		return R.inside(P.x, P.y); 
	}

	/**
     * Returns the intersection of the two rectangles 
     */
	public static Rectangle Intersection (Rectangle R1, Rectangle R2 ) {
		return R1.intersection(R2); 
	}

	/**
     * Check if the rectangle is a square 
     */
	public static boolean IsSquare(Rectangle R) {
		return R.width == R.height;
	}

	/**
     * Returns a the central point for a rectangle 
     */
	public static Point CentralPoint( Rectangle R) {
		return new Point(R.x + (R.width / 2), R.y + (R.height/2));
	}

	/**
     * Returns a bounding rectangle 
     */
	public static Rectangle Bounds( int aLeft, int aTop, int aWidth, int aHeight) {
		return new Rectangle(aLeft,aTop,aWidth,aHeight); 
	}

}

