/*
 * @(#)Str.java	
 *
 * Copyright (c) 1997 Fabula Software GmbH. All Rights Reserved.
 *
 * <a href=http://www.fabula.com>Fabula Software GmbH</a>
 * <a href=mailto:info@fabula.com>eMail</a>
 *
 * 11.08.96 sb begin with the porting of xProcs Delphi unit 
 *
 */



package fabula.util;

/**
 * The class <code>Str</code> defines several methods for 
 * easy string manipulating 
 * <p>
 *
 * @author  <a href="mailto:stefc@fabula.com">Stefan Bther</a>
 * @company <a href="http://www.fabula.com">Fabula Software</a>
 * @version 1.01, 31 Mar 1997
 */

public final class Str {

	// several important ASCII codes 
	public static final char NULL		= 0x0000;
	public static final char BACKSPACE	= 0x0008;
	public static final char TAB		= 0x0009;
	public static final char LF			= 0x000A;
	public static final char CR			= 0x000D;
	public static final char EOF_		= 0x001A;    
	public static final char ESC		= 0x001B;
	public static final char BLANK		= ' ';
	public static final char SPACE		= BLANK;

	public static final byte[] CRLF_	= { 0x0D, 0x0A };
	public static  final String CRLF	= new String(CRLF_,0);

	// digits as chars
	public static final char ZERO      = '0';
	public static final char ONE       = '1';  
	public static final char TWO       = '2';  
	public static final char THREE     = '3';  
	public static final char FOUR      = '4';
	public static final char FIVE      = '5';  
	public static final char SIX       = '6';  
	public static final char SEVEN     = '7';  
	public static final char EIGHT     = '8';  
	public static final char NINE      = '9';

	// special codes 
    public static final char SLASH       = '/';    
    public static final char BACKSLASH   = '\\';
    public static final char HEX_PREFIX  = '$'; 

	// methods 	

	/** builds a easy hashcode for a string */
	public static int Hash(String S ) {
		return S.hashCode();
	}

	/** left substring */
	public static String Left( String S, int Len ) {
		return S.substring(0, Len);
    }

	/** dump a string to the system console */
	public static void Debug( String S ) {
		System.out.println( S ); 
	}

	/** dump a integer to the system console */
	public static void Debug( int i ) { 
		System.out.println( i );
	}

	/** right substring */ 
	public static String Right( String S, int Len ) { 
		int i = S.length();
		if ( Len >= i ) { 
			return S; 
		} else { 
			return S.substring(i-Len,i);
		} 
	}

    /** generate a string with a number of characters */
      public static String Make(char Ch, int Len ) {
      char buf[] = new char[Len];
      for (int i=0; i < (Len-1); i++ ) {
        buf[i] = Ch ; 
      }
      return new String(buf);
    }

    /** generate a string with a number of blanks */
    public static String Blank(int Len ) {
      return Make( BLANK, Len );
    }
	
    /** converts the string to uppercase */
    public static String Upper( String S ) { 
      return S.toUpperCase();
    }

    /** converts the string to lowercase */
    public static String Lower( String S ) {
      return S.toLowerCase();
    }

    /** trim from left */
    public static String TrimL( String S, char Ch ) {
      int len = S.length();
      int st = 0;
      while ((st < len) && (S.charAt(st) == Ch)) {
        st++;
      }
      return ( st > 0 ) ? S.substring(st, len) : S;
    }

    /** trim spaces from left */
    public static String TrimL( String S ) {
      return TrimL( S, BLANK );
    }

    /** trim from right */
    public static String TrimR( String S, char Ch ) {
      int len = S.length();
      int st = 0;
      while ((st < len) && (S.charAt(len-1) == Ch)) {
        len--;
      }
      return ( len < S.length() ) ? S.substring(st, len) : S;
    }

    /** trim spaces from right  */
    public static String TrimR( String S ) {
      return TrimR( S, BLANK ); 
    }

    /** trim all spaces from the left and right */
    public static String Trim( String S ) {
      return S.trim();
    }
     
    /** padding the string from the left */
    public static String PadL( String S, char Ch, int Len ) {
      while ( S.length() < Len ) { 
        S = Ch + S;  
      }
      return S;
    }

    /** padding the string from the left with spaces */
    public static String PadL( String S, int Len ) {
      return PadL( S, BLANK, Len);
    }

    /** padding the string from the left with zeros */
    public static String PadZero( String S, int Len ) {
      return PadL( S, ZERO, Len);
    }

    /** padding the string on the right */
    public static String PadR( String S, char Ch, int Len ) {
      while ( S.length() < Len ) {
        S = S + Ch; 
      }
      return S;
    }

    /** padding the string on the right with spaces */
    public static String PadR( String S, int Len ) {
      return PadR( S, BLANK, Len );
    }


    /** compares two strings */
    public static boolean Equal( String S1, String S2 ) {
      return S1.equals(S2); 
    }

    /** deliver the last character from the string */
    public static char Last( String S ) {
      return S.charAt( S.length() - 1 ); 
    }

    /** add slash character to the string */
    public static String AddSlash( String S ) {
      if ( Last( S ) != SLASH ) {
         return S + SLASH ;
      } else {
         return S; 
      }
    }

    /** remove slash from string */
    public static String DelSlash( String S ) {
      if ( Last( S ) == SLASH ) {
         return S.substring(0, S.length()-1); 
      } else {
         return S ; 
      }
    }
        	        
}
