/*
 * @(#)Sys.java	
 *
 * Copyright (c) 1997 Fabula Software GmbH. All Rights Reserved.
 *
 * <a href=http://www.fabula.com>Fabula Software GmbH</a>
 * <a href=mailto:info@fabula.com>eMail</a>
 */

package fabula.util;

import java.util.Properties;

/**
 * The class <code>Sys</code> defines several methods for accessing 
 * general system attributes and functions 
 * <p>
 *
 * @author  <a href="mailto:stefc@fabula.com">Stefan Bther</a>
 * @company <a href="http://www.fabula.com">Fabula Software</a>
 * @version 1.00, 31 Mar 1997
 */

public class Sys {

	/**
	 * Don't let anyone instantiate this class.
	 */
	private Sys() {}

	private static char BELL	= '\u0007';

	/**
     * Returns a path that is private for the current user and can
	 * be used for temporary files for example. For this purpose 
	 * the system property user.temp is used if this property 
	 * does not exists the property "user.home" is used instead.
     *
     * @return  the path for temporary files
	 */
	public static String TempPath () {
		return System.getProperty("user.temp",System.getProperty("user.home"));
	}

	/**
     * Delays the system for a few milliseconds. 
     *
	 * @param aMs the number of milliseconds the system should be delayed  
	 */
	public static void Delay( int aMs ) {

		long aStart = System.currentTimeMillis();

		while (System.currentTimeMillis()-aStart < aMs ) {
			Thread.yield(); 
		}
	}
	/**
     * Generate a little beep, with using put the ASCII-BELL character 
	 * to the console. 
	 */
	public static void Beep() {
		System.out.print(BELL);
	}

	/**
     * Returns the colordepth of the system. In this case ever 24
	 * until I know how to determine it a little better. 
     *
     * @return  color depth for the system
	 */
	public static int ColorDepth() {
		return 24;
	}

	/**
     * Returns the java version for Example "1.0.1" or "1.1" 
     *
     * @return  Version of the java system 
	 */
	public static String JavaVersion() {
		return System.getProperty("java.version"); 
	}

	/**
     * Returns the name of the operating system Example "Windows NT"
     *
     * @return  name of the operating system 
	 */
	public static String OsName() {
		return System.getProperty("os.name"); 
	}

	/**
     * Returns the version number of the operating system Example "4.0"
     *
     * @return  version of the operating system 
	 */
	public static String OsVersion() {
		return System.getProperty("os.version");
	}
	
	/**
     * Wait for some input 
     *
	 */
	public static String Readln() {
		try { 
			System.in.read(); 
		} 
		catch (Exception ex) 
			{ 
		}
		return "";
	}

	/**
     * dump some values 
     *
	 */
	public static void Writeln() {
		System.out.println();
	}

	public static void Writeln(String S) {
		System.out.println(S);
	}


}